"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDatatableLifecycle = void 0;
const react_1 = __importStar(require("react"));
const data_1 = require("./helpers/data");
const Pagination_1 = __importDefault(require("./Pagination"));
const PaginationOpts_1 = __importDefault(require("./PaginationOpts"));
const TableHeader_1 = __importDefault(require("./TableHeader"));
const TableBody_1 = __importDefault(require("./TableBody"));
const Filter_1 = __importDefault(require("./Filter"));
const object_1 = require("./helpers/object");
const Row_1 = __importDefault(require("react-bootstrap/Row"));
const Button_1 = __importDefault(require("react-bootstrap/Button"));
const Table_1 = __importDefault(require("react-bootstrap/Table"));
const Col_1 = __importDefault(require("react-bootstrap/Col"));
const ButtonGroup_1 = __importDefault(require("react-bootstrap/ButtonGroup"));
const FontAwesome_1 = __importDefault(require("./components/FontAwesome"));
const TableContext_1 = require("./components/TableContext");
/**
 * Datatable lifecycle convenient function.
 * It will be used when we are extending the table.
 **/
function useDatatableLifecycle({ initialSort, onSort, onFilter, rowsPerPage, rowsPerPageOption, async, tableHeaders, classes, tableBody, labels, Components, onRowClick }) {
    (0, react_1.useEffect)(() => {
        // If in development, warn if async and onSort/onFilter are both passed.
        if (process.env.NODE_ENV === 'development') {
            if (async !== undefined) {
                // Warn if onSort and/or onFilter is/are also passed down.
                let str = [];
                if (onSort !== undefined) {
                    str.push('[onSort]');
                }
                if (onFilter !== undefined) {
                    str.push('[onFilter]');
                }
                if (str.length > 0) {
                    console.warn(`You are passing [async] props along with ${(0, object_1.customJoin)(str, ' and ')}. When [async] is enabled, you should not pass onFilter or onSort.`);
                }
                // Warn if all async options are not passed.
                str = [];
                if (async.onFilter === undefined) {
                    str.push('[async.onFilter]');
                }
                if (async.onSort === undefined) {
                    str.push('[async.onSort]');
                }
                if (async.onPaginate === undefined) {
                    str.push('[async.onPaginate]');
                }
                if (str.length > 0) {
                    console.warn(`These async props are missing: ${(0, object_1.customJoin)(str, ', ', 'and ')}`);
                }
            }
        }
    }, []);
    const [state, setState] = (0, react_1.useState)(() => {
        const sortObj = initialSort || {};
        let filterable = async !== undefined && async.onFilter !== undefined;
        let defaultSort = {};
        tableHeaders.forEach((header) => {
            if (header.prop === sortObj.prop) {
                if (header.sortable) {
                    defaultSort = {
                        isAscending: true,
                        prop: header.prop
                    };
                }
            }
            if (header.filterable && async === undefined && !filterable) {
                filterable = true;
            }
        });
        // Define initial state.
        return {
            sortedProp: defaultSort,
            filterable,
            rowsPerPage,
            currentPage: 1,
            filterText: ''
        };
    });
    (0, react_1.useEffect)(() => {
        // Resets the table if the data passed down is different.
        if (tableBody !== undefined) {
            setState((oldState) => (Object.assign(Object.assign({}, oldState), { filterText: '', currentPage: 1, rowsPerPage: rowsPerPage })));
        }
    }, [tableBody, rowsPerPage]);
    const onChangeFilter = (0, react_1.useCallback)((text) => {
        if (async && async.onFilter) {
            async.onFilter(text);
        }
        else {
            setState((oldState) => (Object.assign(Object.assign({}, oldState), { filterText: text, currentPage: 1 })));
        }
    }, [async]);
    const onPageNavigate = (0, react_1.useCallback)((nextPage) => {
        if (async && async.onPaginate) {
            async.onPaginate(nextPage);
        }
        else {
            setState((oldState) => (Object.assign(Object.assign({}, oldState), { currentPage: nextPage })));
        }
    }, [async]);
    const onRowsPerPageChange = (0, react_1.useCallback)((numOfPage) => {
        if (async && async.onRowsPerPageChange) {
            async.onRowsPerPageChange(numOfPage);
        }
        else {
            setState((oldState) => (Object.assign(Object.assign({}, oldState), { rowsPerPage: numOfPage, currentPage: 1 })));
        }
    }, [async]);
    const onSortChange = (0, react_1.useCallback)((nextProp) => {
        if (async && async.onSort) {
            async.onSort(nextProp);
        }
        else {
            setState((oldState) => {
                const nextSort = oldState.sortedProp;
                if (nextProp !== oldState.sortedProp.prop) {
                    nextSort.prop = nextProp;
                    nextSort.isAscending = true;
                }
                else {
                    nextSort.isAscending = !oldState.sortedProp.isAscending;
                }
                return Object.assign(Object.assign({}, oldState), { sortedProp: nextSort });
            });
        }
    }, [async]);
    let data = tableBody;
    let maxPage;
    if (async === undefined) {
        data = (0, data_1.filterData)(tableBody, tableHeaders, state.filterText, onFilter);
        data = (0, data_1.sortData)(data, state.sortedProp, onSort);
        if (state.rowsPerPage) {
            // Pagination needs.
            data = (0, data_1.paginateData)(data, state.currentPage, state.rowsPerPage);
            maxPage = Math.ceil(tableBody.length / state.rowsPerPage);
        }
    }
    else {
        maxPage = async.maxPage;
    }
    const tableClass = (0, object_1.makeClasses)(`table-datatable__root`, classes === null || classes === void 0 ? void 0 : classes.table);
    // Default components.
    // If context has keys, then use context. Instead, use Components props.
    const context = (0, TableContext_1.useComponentProvider)();
    const passedComponents = Object.keys(context).length > 0 ? context : Components;
    let usedComponents = {
        // Global.
        Row: Row_1.default,
        Col: Col_1.default,
        Button: Button_1.default,
        // Table.
        Table: Table_1.default,
        TableHead: 'thead',
        TableBody: 'tbody',
        TableRow: 'tr',
        TableCell: 'td',
        // Filter.
        FilterGroup: undefined,
        // Pagination.
        ButtonGroup: ButtonGroup_1.default,
        // Pagination options.
        PaginationOptsGroup: undefined,
        // ICons.
        SortIcon: FontAwesome_1.default
    };
    if (passedComponents !== undefined) {
        const keys = Object.keys(passedComponents);
        for (const key of keys) {
            // Replace usedComponent fields with the passedComponents fields.
            usedComponents[key] = passedComponents[key];
        }
    }
    return {
        data,
        tableHeaders,
        onChangeFilter,
        onPageNavigate,
        classes,
        onRowsPerPageChange,
        onSortChange,
        tableClass,
        labels,
        rowsPerPageOption,
        Components: usedComponents,
        onRowClick,
        // States.
        filterable: state.filterable,
        filterText: async ? async.filterText : state.filterText,
        rowsPerPage: async ? async.rowsPerPage : state.rowsPerPage,
        currentPage: async ? async.currentPage : state.currentPage,
        sortedProp: async ? async.sortedProp : state.sortedProp,
        maxPage
    };
}
exports.useDatatableLifecycle = useDatatableLifecycle;
/** Datatable Component. */
function Datatable(props) {
    var _a;
    const { data, rowsPerPageOption, tableHeaders, onChangeFilter, onPageNavigate, classes, onRowsPerPageChange, onSortChange, tableClass, labels, filterable, filterText, rowsPerPage, currentPage, sortedProp, maxPage, Components, onRowClick } = useDatatableLifecycle(props);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(Components.Row, { className: (0, object_1.makeClasses)('controlRow__root', classes === null || classes === void 0 ? void 0 : classes.controlRow) },
            react_1.default.createElement(Components.Col, { xs: 12, lg: 4, className: (0, object_1.makeClasses)('d-flex justify-content-start align-items-end', classes === null || classes === void 0 ? void 0 : classes.filterCol) },
                react_1.default.createElement(Filter_1.default, { filterable: filterable, classes: classes, placeholder: (_a = labels === null || labels === void 0 ? void 0 : labels.filter) === null || _a === void 0 ? void 0 : _a.filterPlaceholder, onChangeFilter: onChangeFilter, filterText: filterText, CustomFilterGroup: Components.FilterGroup })),
            react_1.default.createElement(Components.Col, { xs: 12, sm: 4, lg: 4, className: (0, object_1.makeClasses)('d-flex justify-content-lg-center justify-content-xs-start align-items-end', classes === null || classes === void 0 ? void 0 : classes.paginationOptsCol) },
                react_1.default.createElement(PaginationOpts_1.default, { classes: classes, labels: labels, onRowsPerPageChange: onRowsPerPageChange, rowsPerPage: rowsPerPage, rowsPerPageOption: rowsPerPageOption, CustomPaginationOptsGroup: Components.PaginationOptsGroup })),
            react_1.default.createElement(Components.Col, { xs: 12, sm: 8, lg: 4, className: (0, object_1.makeClasses)('d-flex justify-content-end align-items-end', classes === null || classes === void 0 ? void 0 : classes.paginationCol) },
                react_1.default.createElement(Pagination_1.default, { maxPage: maxPage, classes: classes, rowsPerPage: rowsPerPage, currentPage: currentPage, onPageNavigate: onPageNavigate, labels: labels, components: {
                        Button: Components.Button,
                        ButtonGroup: Components.ButtonGroup
                    } }))),
        react_1.default.createElement(Components.Row, null,
            react_1.default.createElement(Components.Col, { xs: 12 },
                react_1.default.createElement(Table_1.default, { className: tableClass },
                    react_1.default.createElement(TableHeader_1.default, { classes: classes, tableHeaders: tableHeaders, sortedProp: sortedProp, onSortChange: onSortChange, components: {
                            TableHead: Components.TableHead,
                            TableCell: Components.TableCell,
                            TableRow: Components.TableRow
                        } }),
                    react_1.default.createElement(TableBody_1.default, { classes: classes, tableHeaders: tableHeaders, labels: labels, data: data, components: {
                            TableBody: Components.TableBody,
                            TableCell: Components.TableCell,
                            TableRow: Components.TableRow
                        }, onRowClick: onRowClick }))))));
}
// Only update if rowsPerPage, rowsPerPageOption, and tableBody changes.
exports.default = react_1.default.memo(Datatable, object_1.shouldTableUpdate);
